#include "GLCD.h"
#include "utilities.h"
void GLCD_init(void)
{
		SET(GLCD_control_dir,GLCD_RS);//
		SET(GLCD_control_dir,GLCD_RW);
		SET(GLCD_control_dir,GLCD_EN);
		SET(GLCD_control_dir,GLCD_RST);
		SET(GLCD_control_dir,GLCD_CS1);
		SET(GLCD_control_dir,GLCD_CS2);
		_BIT_SET(GLCD_data_dir,(0xFF<<datashift));		
		
	  SET(GLCD_control_set,GLCD_RST);
		SET(GLCD_control_clr,GLCD_EN);
		
	// Select the Page0/Page1 and Turn on the GLCD 
    GLCD_SelectPage0();
    GLCD_send_command(0x3F);//LCD ON
		GLCD_send_command(0xC0);//START FROM LEFTMOST
		GLCD_send_command(0x8B);//LINE 0
		GLCD_send_command(0x40);//CURSOR 0
		
    GLCD_SelectPage1();
    GLCD_send_command(0x3F);//LCD ON
		GLCD_send_command(0xC0);//START FROM LEFTMOST
		GLCD_send_command(0x8B);//LINE 0
		GLCD_send_command(0x40);//CURSOR 0
}

void delay_us(int count)
{
  int j=0,i=0;

  for(j=0;j<count;j++)
  {
    /* At 60Mhz, the below loop introduces
    delay of APPROX 1 us */
    for(i=0;i<4;i++);
  }
}

void GLCD_busy(void)
{
	CLR(GLCD_data_dir,BUSY_bit);//SET BUSY_BIT AS INPUT
	delay_us(2);
	SET(GLCD_control_set,GLCD_RW);//SET RW FOR READ MODE
	SET(GLCD_control_clr,GLCD_RS);//CLR RS FOR COMMAND MODE
	delay_us(2);
	SET(GLCD_control_set,GLCD_EN);
	delay_us(2);
	while(CHK(GLCD_data_pin,BUSY_bit));//WAIT FOR INTERNAL OPERATION
	SET(GLCD_control_clr,GLCD_EN);
	SET(GLCD_data_dir,BUSY_bit);
}

void GLCD_send_command(unsigned char command)
{		
	 GLCD_busy();//WAIT
	 SET(GLCD_control_clr,GLCD_RS);
	 SET(GLCD_control_clr,GLCD_RW);
	 _BIT_SET(GLCD_data_clr,(0xff<<datashift));
	 _BIT_SET(GLCD_data_set,(command<<datashift));
	 delay_us(5);
	 SET(GLCD_control_set,GLCD_EN);
	 delay_us(5);
	 SET(GLCD_control_clr,GLCD_EN);
}

void GLCD_send_data(unsigned char data)
{	  
	 GLCD_busy();//WAIT
	 SET(GLCD_control_set,GLCD_RS);
	 SET(GLCD_control_clr,GLCD_RW);
	 _BIT_SET(GLCD_data_clr,(0xff<<datashift));
	 _BIT_SET(GLCD_data_set,(data<<datashift));
	 delay_us(5);
	 SET(GLCD_control_set,GLCD_EN);
	 delay_us(5);
	 SET(GLCD_control_clr,GLCD_EN);	
		
}

void GLCD_DisplayChar(unsigned char *ptr_array,unsigned int start, unsigned int end)
{
	unsigned int i;
    for(i=start;i<end;i++) // 5x7 font, 5 chars + 1 blankspace
        GLCD_send_data(ptr_array[i]);
}





void GLCD_Clear()
{
	int line,cursor;

	for(line=0;line<8;line++)  /* loop through all the 8lines to clear the display */
	{
			GLCD_SelectPage0();/* Select page0 and set the cursor to first position */
		 GLCD_send_command(0x40);
		 GLCD_send_command(0xB8+line);
		for(cursor=0;cursor<128;cursor++) /* Clear all the 128 pixels of selected line */
		{
			if(cursor==64)  /* Has the cursor reached end of page0 */
			{
					GLCD_SelectPage1();	/*  then set it to beginning of page1 */
				GLCD_send_command(0x40);
				 GLCD_send_command(0xB8+line);
			}
			GLCD_send_data(0x00); /* Clear each pixel by displaying blank */
		}
	}
}




void GLCD_SetCursor(int lineNumber,int CursorPosition)
{

	int pageNumber;

	if((lineNumber < 8) && (CursorPosition < 128))
	{
		pageNumber = CursorPosition / 64; //Divide the cursor pos by 64 to get the page
		CursorPosition = CursorPosition % 64; // Cusror pos should be within 64(0x3f),
		if(pageNumber==0)  /* Check for page number and set accordingly */
		{
			GLCD_SelectPage0();
		}
		else if(pageNumber==1)
		{
			GLCD_SelectPage1();
		}

		GLCD_send_command(lineNumber+0xB8); /* Command the LCD to move to specified page,line,cursor*/
		GLCD_send_command(CursorPosition+0x40);
	}
}



void GLCD_DisplayLogo(unsigned char ptr_Logo[])
{
	int line,cursor;

	for(line=0;line<8;line++)
	{
		GLCD_SelectPage0();
    GLCD_send_command(0x40);
		GLCD_send_command(0xB8+line);
		for(cursor=0;cursor<128;cursor++)
		{
			if(cursor==64)
			{
				GLCD_SelectPage1();
				GLCD_send_command(0x40);
				GLCD_send_command(0xB8+line);
			}
			GLCD_send_data(*ptr_Logo++);
		}
	}
	delay_us(20);
}


void GLCD_Display(unsigned char ptr_Logo[],int start_line,int end_line,int start_cursor,int end_cursor)
{
	int line,cursor;
	for(line=start_line;line<=end_line;line++)
	{
		if(start_cursor<64)
		{
		GLCD_SelectPage0();
		GLCD_send_command(0xB8+line);
		cursor=start_cursor;
		GLCD_send_command(0x40+cursor);
		for(;cursor<64;++cursor)
		{
			if(cursor>end_cursor)
				break;
			GLCD_send_data(*ptr_Logo++);
		}
	}
		if(end_cursor>=64)
		{
		GLCD_SelectPage1()
		GLCD_send_command(0xB8+line);
		cursor=64;
		for(;cursor<start_cursor;++cursor);	
		GLCD_send_command(0x40+cursor-64);
		for(;cursor<=end_cursor;++cursor)
		{
			GLCD_send_data(*ptr_Logo++);
		}
	}
	}
}
